/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator;

import java.util.List;

/**
 * An interface to describe functions callable from expressions.
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 *
 * @see StandardFunction
 */
public interface Function
{
    /**
     * Invokes the function with the specified argument expressions in
     * the specified evaluation context.
     *
     * @param ec The EvaluationContext in which this function is being called.
     * @param functionName The name of the function being called.
     * @param argumentExpressions The expressions which, when evaluated, result
     *                            in the arguments to the function.
     *
     * @return The result of the function.
     *
     * @throws EvaluationException If there is a problem evaluating any of
     *                             the arguments' expressions, or if there is
     *                             an error during function execution.
     */
    public Object call(EvaluationContext ec,
		       String functionName,
		       List argumentExpressions)
	throws EvaluationException;
}
