/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;

/**
 * <p>
 *    Assigns a value to a symbolic identifier.
 * </p>
 *
 * <p>
 *   Usage: set(id: String, value: Object)
 * </p>
 *
 * <p>
 *    Returns the value to which the identifier was set.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.3 $ $Date: 2008-06-20 22:58:48 $
 */
public class CallFunction
    extends IJWInvocation
{
    public Object call(EvaluationContext ec,
		       String functionName,
		       Object[] arguments)
	throws EvaluationException
    {
	if(null == arguments || 2 > arguments.length)
	    throw new EvaluationException("Expected at least 2 arguments, got "
					  + (null == arguments ? "0" : String.valueOf(arguments.length)));

	functionName = (String)arguments[0];

	Object[] args = new Object[arguments.length - 1];

	System.arraycopy(arguments, 1, args, 0, arguments.length - 1);

	return super.call(ec, functionName, args);
    }
}
/*
  set("f", create("java.awt.Frame", "Hello World"))
  call("setSize", f, 100, 100)
  call("setVisible", f, true)
*/
