/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.StandardFunction;

/**
 * <p>
 *     Returns true if all arguments are null.
 * </p>
 *
 * <p>
 *    This method will typically take a single argument, but you may invoke
 *    it with an arbitrary number of arguments. <code>true</code> is returned
 *    if all arguments are null.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class IsNullFunction
    extends StandardFunction
{
    public Object call(EvaluationContext ec,
		       String functionName,
		       Object[] arguments)
	throws EvaluationException
    {
	if(null == arguments || 0 == arguments.length)
	    throw new EvaluationException("Expected at least one argument");

	boolean isNull = true;

	for(int i=0; i<arguments.length; ++i)
	{
	    if(null != arguments[i])
	    {
		isNull = false;
		break;
	    }
	}

	return isNull ? Boolean.TRUE : Boolean.FALSE;
    }
}
