/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.function;

import java.util.List;

import net.christopherschultz.evaluator.EvaluationContext;
import net.christopherschultz.evaluator.EvaluationException;
import net.christopherschultz.evaluator.Expression;
import net.christopherschultz.evaluator.Function;
import net.christopherschultz.evaluator.IdentifierExpression;
import net.christopherschultz.evaluator.StandardFunction;

/**
 * <p>
 *    Assigns a value to a symbolic identifier.
 * </p>
 *
 * <p>
 *   Usage: set(id: String, value: Object)
 * </p>
 *
 * <p>
 *    Returns the value to which the identifier was set.
 * </p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.5 $ $Date: 2008-06-20 22:58:48 $
 */
public class SetVariableFunction
    implements Function
{
    public Object call(EvaluationContext ec,
                       String functionName,
                       List argumentExpressions)
        throws EvaluationException
    {
        if(null == argumentExpressions || 2 != argumentExpressions.size())
            throw new EvaluationException("Expected 2 arguments, got "
                                          + (null == argumentExpressions ? "0" : String.valueOf(argumentExpressions.size())));

        Expression varNameExp = (Expression)argumentExpressions.get(0);

        // Can only set the values of identifiers / symbols
        Expression exp = varNameExp;

        while(null != exp
              && !(exp instanceof IdentifierExpression))
        {
            Object result = exp.evaluate(ec);

            if(result instanceof Expression)
            {
                exp = (Expression)result;
            }
            else
            {
                exp = null;
            }
        }

        if(null == exp || !(exp instanceof IdentifierExpression))
            throw new EvaluationException("Cannot define a non-identifier: " + varNameExp);

        String varName = (String)((IdentifierExpression)exp).getValue();
        Object value = StandardFunction.getExpressionValue(ec, (Expression)argumentExpressions.get(1));

        ec.set(varName, value);

        return value;
    }
}
