/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator;

import junit.framework.Assert;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import net.christopherschultz.evaluator.util.DefaultEvaluationContext;

/**
 * Unit tests for UnaryMinusExpression.<p>
 *
 * @author Chris Schultz
 * @version $Revision: 1.2 $ $Date: 2008-06-20 22:58:48 $
 */
public class UnaryMinusExpressionTests
    extends TestCase
{
    EvaluationContext ec;

    public void setUp()
    {
	ec = new DefaultEvaluationContext();
    }

    public void testNegationPlus()
	throws EvaluationException
    {
	Expression exp = new UnaryMinusExpression(new ConstantExpression(new Integer(4)));

	Assert.assertEquals("Unary minus doesn't work",
			    new Integer(-4), exp.evaluate(ec));
    }

    public void testNegationMinus()
	throws EvaluationException
    {
	Expression exp = new UnaryMinusExpression(new ConstantExpression(new Integer(-4)));

	Assert.assertEquals("Negation doesn't work",
			    new Integer(4), exp.evaluate(ec));
    }

    public void testNegationByte()
	throws EvaluationException
    {
	Expression exp = new UnaryMinusExpression(new ConstantExpression(new Byte((byte)-4)));

	Assert.assertEquals("Unary minus doesn't produce correct return type",
			    new Byte((byte)4), exp.evaluate(ec));
    }

    public void testNegationShort()
	throws EvaluationException
    {
	Expression exp = new UnaryMinusExpression(new ConstantExpression(new Short((short)-4)));

	Assert.assertEquals("Unary minus doesn't produce correct return type",
			    new Short((short)4), exp.evaluate(ec));
    }

    public void testNegationLong()
	throws EvaluationException
    {
	Expression exp = new UnaryMinusExpression(new ConstantExpression(new Long((long)-4)));

	Assert.assertEquals("Unary minus doesn't produce correct return type",
			    new Long((long)4), exp.evaluate(ec));
    }

    // TODO: What about overflow?
    /*
    public void testOverflow()
	throws EvaluationException
    {
	// Integer.MIN_VALUE=-2147483648
	System.err.println("Integer.MIN_VALUE = " + Integer.MIN_VALUE);
	System.err.println("Integer.MAX_VALUE = " + Integer.MAX_VALUE);
	System.err.println("-Integer.MIN_VALUE = " + (-Integer.MIN_VALUE));
	System.err.println("-Integer.MAX_VALUE = " + (-Integer.MAX_VALUE));
	System.err.println("-(Integer.MIN_VALUE+1) = " + (-(Integer.MIN_VALUE+1)));
	Expression exp = new UnaryMinusExpression(new ConstantExpression(new Integer(Integer.MIN_VALUE)));

	Assert.assertEquals("Unary minus doesn't handle overflow",
			    new Integer(Integer.MAX_VALUE), exp.evaluate(ec));
    }
    */

    public void testUnderflow()
	throws EvaluationException
    {
	Expression exp = new UnaryMinusExpression(new ConstantExpression(new Integer(Integer.MAX_VALUE)));

	Assert.assertEquals("Unary minus doesn't handle underflow",
			    new Integer(-Integer.MAX_VALUE), exp.evaluate(ec));
    }

    public void testNulls()
	throws EvaluationException
    {
	Expression exp = new UnaryMinusExpression(new IdentifierExpression("null"));

	Assert.assertNull("Negating nulls does not return null",
			  exp.evaluate(ec));
    }

    /**
     * Returns a Test which will execute all the test methods found
     * in this class.<p>
     *
     * @return A Test for JUnit to run.
     */
    public static Test suite()
    {
        TestSuite ts = new TestSuite(UnaryMinusExpressionTests.class);

        return ts;
    }

    /**
     * Creates a new UnitTest to execute the specified test.<p>
     *
     * @param name The name of the test method to run.
     */
    public UnaryMinusExpressionTests(String name)
    {
        super(name);
    }
}
