/**
 * This software is Copyright (C) 2008 Christopher Schultz
 *
 * Please see the COPYING file for more information.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package net.christopherschultz.evaluator.util;

import junit.framework.Assert;
import junit.framework.TestCase;

public class ConstantLoaderTests
    extends TestCase
{
    public static final String A_TEST_FIELD = "thisIsATest";

    private ConstantLoader _loader = new ConstantLoader();

    public void testJavaFieldLoading()
	throws Exception
    {
	Assert.assertNull(_loader.getValue("null"));
	Assert.assertEquals(java.lang.Boolean.TRUE,
			    _loader.getValue("java.lang.Boolean#TRUE"));
	Assert.assertEquals(java.lang.Boolean.FALSE,
			    _loader.getValue("java.lang.Boolean#FALSE"));
	Assert.assertEquals(A_TEST_FIELD,
			    _loader.getValue(this.getClass().getName() + "#A_TEST_FIELD"));
    }

    public void testClassSpecificLoading()
	throws Exception
    {
	Assert.assertEquals(new Integer(1),
		      _loader.getValue("java.lang.Integer(1)"));
	Assert.assertEquals(new Integer(2),
		      _loader.getValue("java.lang.Integer(2)"));
	Assert.assertEquals(new Integer(3),
		      _loader.getValue("java.lang.Integer(3)"));
	Assert.assertEquals(new Integer(-4),
		      _loader.getValue("java.lang.Integer(-4)"));
	Assert.assertEquals("just a test",
		      _loader.getValue("java.lang.String(just a test)"));
    }

    public void testAutoBooleanLoading()
	throws Exception
    {
	Assert.assertEquals(Boolean.TRUE,
			    _loader.getValue("true"));
	Assert.assertEquals(Boolean.TRUE,
			    _loader.getValue("tRue"));
	Assert.assertEquals(Boolean.FALSE,
			    _loader.getValue("false"));
	Assert.assertEquals(Boolean.FALSE,
			    _loader.getValue("faLSe"));
    }

    public void testAutoDoubleLoading()
	throws Exception
    {
	Assert.assertEquals(new Double(3.14159),
			    _loader.getValue("3.14159"));
	Assert.assertEquals(new Double(100),
			    _loader.getValue("100d"));
	Assert.assertEquals(new Double(0.5),
			    _loader.getValue("0.5"));
	Assert.assertEquals(new Double(-2),
			    _loader.getValue("-2.0"));
    }

    public void testAutoFloatLoading()
	throws Exception
    {
	Assert.assertEquals(new Float(3.14159),
			    _loader.getValue("3.14159f"));
	Assert.assertEquals(new Float(100),
			    _loader.getValue("100f"));
	Assert.assertEquals(new Float(0.5),
			    _loader.getValue("0.5f"));
	Assert.assertEquals(new Float(-2),
			    _loader.getValue("-2f"));
    }

    public void testAutoLongLoading()
	throws Exception
    {
	Assert.assertEquals(new Long(2145),
			    _loader.getValue("2145l"));
	Assert.assertEquals(new Long(-40),
			    _loader.getValue("-40l"));
	Assert.assertEquals(new Long(908),
			    _loader.getValue("908l"));
    }

    public void testAutoIntegerLoading()
	throws Exception
    {
	Assert.assertEquals(new Integer(2145),
			    _loader.getValue("2145"));
	Assert.assertEquals(new Integer(-40),
			    _loader.getValue("-40"));
	Assert.assertEquals(new Integer(908),
			    _loader.getValue("908"));
    }

    public void testAutoStringLoading()
	throws Exception
    {
	Assert.assertEquals("1.2.3.4",
			    _loader.getValue("1.2.3.4"));
	Assert.assertEquals("Chuck Berry",
			    _loader.getValue("Chuck Berry"));
	Assert.assertEquals("-1.0q",
			    _loader.getValue("-1.0q"));
    }

    public void testExplicitStringLoading()
	throws Exception
    {
	Assert.assertEquals("1.0",
			    _loader.getValue("'1.0'"));
	Assert.assertEquals("null",
			    _loader.getValue("\"null\""));
	Assert.assertEquals("java.lang.Integer(1)",
			    _loader.getValue("\"java.lang.Integer(1)\""));
    }

    public void testBlankStrings()
    {
	Assert.assertEquals("", _loader.getValue(""));

	_loader.setProperty(_loader.BLANK_NULL, Boolean.TRUE);

	Assert.assertNull(_loader.getValue(""));
    }
}
